Package: sirt
Type: Package
Title: Supplementary Item Response Theory Models
Version: 4.2-133
Date: 2025-09-27 12:57:51
Authors@R: c( person(given = "Alexander",
                    family = "Robitzsch",
                    role = c("aut", "cre"),
                    email = "robitzsch@ipn.uni-kiel.de",
                    comment = c(ORCID = "0000-0002-8226-3132")) )
Maintainer: Alexander Robitzsch <robitzsch@ipn.uni-kiel.de>
Description: 
    Supplementary functions for item response models aiming
    to complement existing R packages. The functionality includes among others
    multidimensional compensatory and noncompensatory IRT models
    (Reckase, 2009, <doi:10.1007/978-0-387-89976-3>), 
    MCMC for hierarchical IRT models and testlet models
    (Fox, 2010, <doi:10.1007/978-1-4419-0742-4>), 
    NOHARM (McDonald, 1982, <doi:10.1177/014662168200600402>), 
    Rasch copula model (Braeken, 2011, <doi:10.1007/s11336-010-9190-4>;
    Schroeders, Robitzsch & Schipolowski, 2014, <doi:10.1111/jedm.12054>),
    faceted and hierarchical rater models (DeCarlo, Kim & Johnson, 2011,
    <doi:10.1111/j.1745-3984.2011.00143.x>),
    ordinal IRT model (ISOP; Scheiblechner, 1995, <doi:10.1007/BF02301417>), 
    DETECT statistic (Stout, Habing, Douglas & Kim, 1996, 
    <doi:10.1177/014662169602000403>), local structural equation modeling 
    (LSEM; Hildebrandt, Luedtke, Robitzsch, Sommer & Wilhelm, 2016,
    <doi:10.1080/00273171.2016.1142856>).
Depends: R (>= 3.5)
Imports: CDM, graphics, methods, parallel, pbapply, Rcpp, stats, TAM,
        utils
Suggests: coda, igraph, lavaan, MASS, Matrix, miceadds, minqa, mirt,
        mvtnorm, nloptr, optimx, pbivnorm, pbv, psych, sfsmisc, sm,
        survey
LinkingTo: pbv, Rcpp, RcppArmadillo
URL: https://github.com/alexanderrobitzsch/sirt,
        https://sites.google.com/view/alexander-robitzsch/software
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-09-27 11:06:19 UTC; sunpn563
Author: Alexander Robitzsch [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8226-3132>)
Repository: CRAN
Date/Publication: 2025-09-27 23:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:23:00 UTC; unix
Archs: sirt.so.dSYM
