

useDynLib(sirt)



####################################################
################## IMPORTS #########################
####################################################

#***************************************************
#*************** R Basics / Linking To *************
#***************************************************

importFrom(graphics, abline)
importFrom(graphics, dotchart)
importFrom(graphics, lines)
importFrom(graphics, matplot)
importFrom(graphics, par)
importFrom(graphics, plot)
importFrom(graphics, points)
importFrom(graphics, polygon)
importFrom(graphics, rug)
importFrom(graphics, segments)
importFrom(graphics, text)
importFrom(methods, is)
importFrom(Rcpp, evalCpp)
importFrom(stats, acf)
importFrom(stats, aggregate)
importFrom(stats, anova)
importFrom(stats, as.dist)
importFrom(stats, as.formula)
importFrom(stats, coef)
importFrom(stats, confint)
importFrom(stats, cor)
importFrom(stats, cov)
importFrom(stats, cov2cor)
importFrom(stats, cov.wt)
importFrom(stats, cutree)
importFrom(stats, dbeta)
importFrom(stats, dnorm)
importFrom(stats, fitted)
importFrom(stats, glm)
importFrom(stats, hclust)
importFrom(stats, ksmooth)
importFrom(stats, lm)
importFrom(stats, lm.wfit)
importFrom(stats, logLik)
importFrom(stats, mad)
importFrom(stats, median)
importFrom(stats, medpolish)
importFrom(stats, model.matrix)
importFrom(stats, na.omit)
importFrom(stats, optim)
importFrom(stats, optimize)
importFrom(stats, p.adjust)
importFrom(stats, pchisq)
importFrom(stats, plogis)
importFrom(stats, pnorm)
importFrom(stats, predict)
importFrom(stats, promax)
importFrom(stats, qlogis)
importFrom(stats, qnorm)
importFrom(stats, qt)
importFrom(stats, quantile)
importFrom(stats, rbeta)
importFrom(stats, rect.hclust)
importFrom(stats, resid)
importFrom(stats, residuals)
importFrom(stats, rgamma)
importFrom(stats, rnorm)
importFrom(stats, runif)
importFrom(stats, sd)
importFrom(stats, spline)
importFrom(stats, var)
importFrom(stats, vcov)
importFrom(stats, weighted.mean)
importFrom(utils, combn)
importFrom(utils, flush.console)
importFrom(utils, packageDescription)
importFrom(utils, read.table)
importFrom(utils, write.table)



#***************************************************
#********** Imports (own packages) *****************
#***************************************************

importFrom(CDM, CDM_rmvnorm)
importFrom(CDM, csink)
importFrom(CDM, IRT.anova)
importFrom(CDM, IRT.compareModels)
importFrom(CDM, IRT.data)
importFrom(CDM, IRT.derivedParameters)
importFrom(CDM, IRT.expectedCounts)
importFrom(CDM, IRT.factor.scores)
importFrom(CDM, IRT.IC)
importFrom(CDM, IRT.irfprob)
importFrom(CDM, IRT.irfprobPlot)
importFrom(CDM, IRT.itemfit)
importFrom(CDM, IRT.jackknife)
importFrom(CDM, IRT.likelihood)
importFrom(CDM, IRT.modelfit)
importFrom(CDM, IRT.parameterTable)
importFrom(CDM, IRT.posterior)
importFrom(CDM, IRT.predict)
importFrom(CDM, IRT.repDesign)
importFrom(CDM, IRT.se)
importFrom(CDM, IRT_RMSD_calc_rmsd)
importFrom(CDM, modelfit.cor2)
importFrom(CDM, numerical_gradient)
importFrom(CDM, numerical_Hessian)
importFrom(CDM, osink)
importFrom(TAM, IRT.drawPV)
importFrom(TAM, IRT.informationCurves)
importFrom(TAM, IRT.linearCFA)
importFrom(TAM, IRT.residuals)
importFrom(TAM, IRT.simulate)
importFrom(TAM, IRT.threshold)
importFrom(TAM, IRT.truescore)
importFrom(TAM, IRT.WrightMap)
importFrom(TAM, lavaanify.IRT)
importFrom(TAM, require_namespace_msg)
importFrom(TAM, tam.modelfit)
importFrom(TAM, tam.ctt3)
importFrom(TAM, tam_matrix2)
importFrom(TAM, tam_outer)
importFrom(TAM, tam_packageinfo)
importFrom(TAM, tam_print_call)
importFrom(TAM, tam_rsessinfo)
importFrom(TAM, weighted_mean)
importFrom(TAM, weighted_sd)



#***************************************************
#************** Imports ****************************
#***************************************************

importFrom(parallel, clusterEvalQ)
importFrom(parallel, clusterExport)
importFrom(parallel, detectCores)
importFrom(parallel, makeCluster)
importFrom(parallel, parSapply)
importFrom(parallel, stopCluster)
importFrom(pbapply, pblapply)



#***************************************************
#************** Suggests ***************************
#***************************************************
# requireNamespace("another_package")
# TAM::require_namespace_msg("another_package")

# importFrom(coda, as.mcmc.list)
# importFrom(coda, effectiveSize)
# importFrom(coda, mcmc)
# importFrom(igraph, graph.edgelist)
# importFrom(lavaan, cfa)
# importFrom(lavaan, fitMeasures)
# importFrom(lavaan, lavaan)
# importFrom(lavaan, lavaanify)
# importFrom(lavaan, parameterEstimates)
# importFrom(lavaan, parameterTable)
# importFrom(lavaan, standardizedSolution)
# importFrom(lavaan, sem)
# importFrom(lavaan.survey, lavaan.survey)
# importFrom(MASS, ginv)
# importFrom(Matrix, bdiag)
# importFrom(mirt, extract.item)
# importFrom(mirt, mirt)
# importFrom(mirt, itemplot)
# importFrom(mirt, mirt.model)
# importFrom(mirt, probtrace)
# importFrom(mvtnorm, dmvnorm)
# importFrom(mvtnorm, pmvnorm)
# importFrom(mvtnorm, rmvnorm) # replaced by CDM::CDM_rmvnorm
# importFrom(pbivnorm, pbivnorm)
# importFrom(psych, cor.smooth)
# importFrom(psych, fa)
# importFrom(psych, omega)
# importFrom(sfsmisc, QUnif)
# importFrom(sm, h.select)
# importFrom(sm, sm.binomial)
# importFrom(survey, svydesign)


####################################################
################## EXPORTS #########################
####################################################

export(automatic.recode)
export(bounds_parameters)
export(brm.irf)
export(brm.sim)
export(btm)
export(btm_sim)
export(categorize)
export(ccov.np)
export(cfa_meas_inv)
export(class.accuracy.rasch)
export(colCumsums.sirt)
export(conf.detect)
export(data.wide2long)
export(decategorize)
export(detect.index)
export(dexppow)
export(dif.logistic.regression)
export(dif.strata.variance)
export(dif.variance)
export(dimproper)
export(dinvgamma2)
export(dirichlet.mle)
export(dirichlet.simul)
export(dmlavaan)
export(eigenvalues.manymatrices)
export(equating.rasch)
export(equating.rasch.jackknife)
export(expl.detect)
export(f1d.irt)
export(fit.adisop)
export(fit.isop)
export(fuzcluster)
export(fuzdiscr)
export(genlogis.moments)
export(ginverse_sym)
export(gom.em)
export(gom.jml)
export(greenyang.reliability)
export(hard_thresholding)
export(invariance_alignment_cfa_config)
export(invariance_alignment_constraints)
export(invariance_alignment_simulate)
export(invariance.alignment)
export(IRT.mle)
export(isop.dich)
export(isop.poly)
export(isop.scoring)
export(isop.test)
export(L0_polish)
export(latent.regression.em.normal)
export(latent.regression.em.raschtype)
export(lavaan2mirt)
export(lc.2raters)
export(likelihood.adjustment)
export(linking_haberman_itempars_convert)
export(linking_haberman_itempars_prepare)
export(linking.haberman)
export(linking.haberman.lq)
export(linking.haebara)
export(linking.robust)
export(linking_2groups)
export(locpolycor)
export(lq_fit)
export(lq_fit_estimate_power)
export(lsdm)
export(lsem.bootstrap)
export(lsem.estimate)
export(lsem.MGM.stepfunctions)
export(lsem.permutationTest)
export(lsem.test)
export(lsem_local_weights)
export(marginal.truescore.reliability)
export(mcmc.2pno)
export(mcmc.2pno.ml)
export(mcmc.2pnoh)
export(mcmc.3pno.testlet)
export(mcmc.list.descriptives)
export(mcmc_coef)
export(mcmc_confint)
export(mcmc_derivedPars)
export(mcmc_plot)
export(mcmc_Rhat)
export(mcmc_summary)
export(mcmc_vcov)
export(mcmc_WaldTest)
export(mcmclist2coda)
export(md.pattern.sirt)
export(mirt_summary)
export(mirt.specify.partable)
export(mirt.wrapper.coef)
export(mirt.wrapper.fscores)
export(mirt.wrapper.itemplot)
export(mirt.wrapper.posterior)
export(mle.pcm.group)
export(modelfit.cor.poly)
export(modelfit.sirt)
export(monoreg.colwise)
export(monoreg.rowwise)
export(move_variables_df)
export(nedelsky.irf)
export(nedelsky.latresp)
export(nedelsky.sim)
export(noharm.sirt)
export(np.dich)
export(parmsummary_extend)
export(pbivnorm2)
export(pcm.conversion)
export(pcm.fit)
export(person.parameter.rasch.copula)
export(personfit.stat)
export(pgenlogis)
export(plausible.value.imputation.raschtype)
export(polychoric2)
export(pow)
export(predict_scale_group_means)
export(print_digits)
export(prior_model_parse)
export(prmse.subscores.scales)
export(prob.guttman)
export(Q3)
export(Q3.testlet)
export(qmc.nodes)
export(R2conquest)
export(R2noharm)
export(R2noharm.EAP)
export(R2noharm.jackknife)
export(rasch.conquest)		# defunct
export(rasch.copula2)
export(rasch.copula3)
export(rasch.evm.pcm)
export(rasch.jml)
export(rasch.jml.biascorr)
export(rasch.jml.jackknife1)
export(rasch.mirtlc)
export(rasch.mml2)
export(rasch.pairwise)
export(rasch.pairwise.itemcluster)
export(rasch.pml2)		# defunct
export(rasch.pml3)
export(rasch.prox)
export(rasch.va)
export(read.multidimpv)
export(read.pimap)
export(read.pv)
export(read.show)
export(read.show.regression)
export(read.show.term)
export(reliability.nonlinearSEM)
export(resp_groupwise)
export(rexppow)
export(rinvgamma2)
export(rm.facets)
export(rm.sdt)
export(rm_proc_data)
export(rmvn)
export(rowCumsums.sirt)
export(rowIntervalIndex.sirt)
export(rowKSmallest.sirt)
export(rowKSmallest2.sirt)
export(rowMaxs.sirt)
export(rowMins.sirt)
export(ruvn)
export(scale_group_means)
export(soft_thresholding)
export(sia.sirt)
export(sim.qm.ramsay)
export(sim.rasch.dep)
export(sim.raschtype)
export(sirt_abs_smooth)
export(sirt_antifisherz)
export(sirt_attach_list_elements)
export(sirt_colMaxs)
export(sirt_colMeans)
export(sirt_colMedians)
export(sirt_colMins)
export(sirt_colSDs)
export(sirt_dnorm_discrete)
export(sirt_eigenvalues)
export(sirt_fisherz)
export(sirt_matrix2)
export(sirt_optimizer)
export(sirt_permutations)
export(sirt_rbind_fill)
export(sirt_rcpp_discrete_inverse)
export(sirt_rcpp_polychoric2)
export(sirt_sum_norm)
export(sirt_summary_print_call)
export(sirt_summary_print_objects)
export(sirt_summary_print_package_rsession)
export(sirt_summary_print_package)
export(sirt_summary_print_rsession)
export(smirt)
export(stratified.cronbach.alpha)
export(summary.R2noharm)
export(tam2mirt)
export(testlet.marginalized)
export(testlet.yen.q3)		# defunct
export(tetrachoric2)
export(tracemat)
export(truescore.irt)
export(unidim.test.csn)
export(wle.rasch)
export(wle.rasch.jackknife)
export(xxirt)
export(xxirt_createDiscItem)
export(xxirt_createParTable)
export(xxirt_createThetaDistribution)
export(xxirt_hessian)
export(xxirt_modifyParTable)
export(xxirt_sandwich_pml)
export(yen.q3)		# defunct
    





################################################
################ S3 methods ####################
################################################

S3method(anova, gom)
S3method(anova, prob.guttman)
S3method(anova, rasch.copula2)
S3method(anova, rasch.copula3)
S3method(anova, rasch.mirtlc)
S3method(anova, rasch.mml)
S3method(anova, rm.facets)
S3method(anova, rm.sdt)
S3method(anova, smirt)
S3method(anova, xxirt)
S3method(coef, rasch.evm.pcm)
S3method(coef, xxirt)
S3method(confint, xxirt)
S3method(IRT.expectedCounts, MultipleGroupClass)
S3method(IRT.expectedCounts, rasch.mml)
S3method(IRT.expectedCounts, SingleGroupClass)
S3method(IRT.expectedCounts, xxirt)
S3method(IRT.factor.scores, rm.facets)
S3method(IRT.factor.scores, rm.sdt)
S3method(IRT.factor.scores, xxirt)
S3method(IRT.irfprob, gom)
S3method(IRT.irfprob, prob.guttman)
S3method(IRT.irfprob, rasch.mirtlc)
S3method(IRT.irfprob, rasch.mml)
S3method(IRT.irfprob, rm.facets)
S3method(IRT.irfprob, rm.sdt)
S3method(IRT.irfprob, MultipleGroupClass)
S3method(IRT.irfprob, SingleGroupClass)
S3method(IRT.irfprob, smirt)
S3method(IRT.irfprob, xxirt)
S3method(IRT.likelihood, gom)
S3method(IRT.likelihood, rasch.copula2)
S3method(IRT.likelihood, rasch.copula3)
S3method(IRT.likelihood, rasch.mirtlc)
S3method(IRT.likelihood, rasch.mml)
S3method(IRT.likelihood, rm.facets)
S3method(IRT.likelihood, MultipleGroupClass)
S3method(IRT.likelihood, SingleGroupClass)
S3method(IRT.likelihood, smirt)
S3method(IRT.likelihood, xxirt)
S3method(IRT.modelfit, gom)
S3method(IRT.modelfit, rasch.mirtlc)
S3method(IRT.modelfit, rasch.mml)
S3method(IRT.modelfit, rm.facets)
S3method(IRT.modelfit, rm.sdt)
S3method(IRT.modelfit, smirt)
S3method(IRT.modelfit, sirt)
S3method(IRT.modelfit, xxirt)
S3method(IRT.posterior, gom)
S3method(IRT.posterior, prob.guttman)
S3method(IRT.posterior, rasch.copula2)
S3method(IRT.posterior, rasch.copula3)
S3method(IRT.posterior, rasch.mirtlc)
S3method(IRT.posterior, rasch.mml)
S3method(IRT.posterior, rm.facets)
S3method(IRT.posterior, rm.sdt)
S3method(IRT.posterior, MultipleGroupClass)
S3method(IRT.posterior, SingleGroupClass)
S3method(IRT.posterior, smirt)
S3method(IRT.posterior, xxirt)
S3method(IRT.se, xxirt)
S3method(logLik, gom)
S3method(logLik, prob.guttman)
S3method(logLik, rasch.copula2)
S3method(logLik, rasch.copula3)
S3method(logLik, rasch.mirtlc)
S3method(logLik, rasch.mml)
S3method(logLik, rm.facets)
S3method(logLik, rm.sdt)
S3method(logLik, smirt)
S3method(logLik, xxirt)
S3method(plot, isop)
S3method(plot, linking.robust)
S3method(plot, lsem)
S3method(plot, lsem.permutationTest)
S3method(plot, lsdm)
S3method(plot, mcmc.sirt)
S3method(plot, np.dich)
S3method(plot, rasch.mml)
S3method(plot, rm.sdt)
S3method(predict, btm)
S3method(summary, btm)
S3method(summary, conf.detect)
S3method(summary, fuzcluster)
S3method(summary, gom)
S3method(summary, gom.jml)
S3method(summary, invariance_alignment_constraints)
S3method(summary, invariance.alignment)
S3method(summary, IRT.modelfit.gom)
S3method(summary, IRT.modelfit.rasch.mirtlc)
S3method(summary, IRT.modelfit.rasch.mml)
S3method(summary, IRT.modelfit.rm.facets)
S3method(summary, IRT.modelfit.rm.sdt)
S3method(summary, IRT.modelfit.smirt)
S3method(summary, isop)
S3method(summary, isop.test)
S3method(summary, latent.regression)
S3method(summary, lc.2raters)
S3method(summary, linking.haberman)
S3method(summary, linking.haberman.lq)
S3method(summary, linking.haebara)
S3method(summary, linking.robust)
S3method(summary, lsdm)
S3method(summary, lsem)
S3method(summary, lsem.permutationTest)
S3method(summary, mcmc.sirt)
S3method(summary, mcmc_WaldTest)
S3method(summary, modelfit.sirt)
S3method(summary, noharm.sirt)
S3method(summary, prob.guttman)
S3method(summary, R2conquest)
S3method(summary, R2noharm)
S3method(summary, R2noharm.jackknife)
S3method(summary, rasch.copula2)
S3method(summary, rasch.copula3)
S3method(summary, rasch.evm.pcm)
S3method(summary, rasch.jml)
S3method(summary, rasch.mirtlc)
S3method(summary, rasch.mml)
S3method(summary, rasch.pairwise)
S3method(summary, rasch.pml)
S3method(summary, rm.facets)
S3method(summary, rm.sdt)
S3method(summary, smirt)
S3method(summary, xxirt)
S3method(vcov, rasch.evm.pcm)
S3method(vcov, xxirt)
