# sirt
#### Supplementary Item Response Theory Models


If you use `sirt` and have suggestions for improvement or have found bugs, please email me at robitzsch@leibniz-ipn.de.
Please always provide a minimal dataset, necessary to demonstrate the problem, 
a minimal runnable code necessary to reproduce the issue, which can be run on the given dataset, and
all necessary information on the used librarys, the R version, and the OS it is run on, perhaps a sessionInfo().


#### CRAN version

<!---
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version-last-release/sirt)](https://cran.r-project.org/package=sirt)
&#160;&#160;
-->

The official version of `sirt` is hosted on CRAN and may be found [here](https://cran.r-project.org/package=sirt). 
The CRAN version can be installed from within R using:

```r
utils::install.packages("sirt")
```

#### GitHub version

The version hosted [here](https://github.com/alexanderrobitzsch/sirt) is the development version of `sirt`. 
The GitHub version can be installed using `devtools` as:

```r
devtools::install_github("alexanderrobitzsch/sirt")
```
