## File Name: summary.rasch.copula3.R
## File Version: 0.131


#* summary for rasch.copula object
summary.rasch.copula3 <- function( object, file=NULL, digits=3, ...)
{
    summary.rasch.copula2( object=object, file=file, digits=digits, ... )
}
