## File Name: rm_calclike.R
## File Version: 0.07


#**** calculation of the likelihood
rm_calclike <- function(dat2, dat2resp, probs,K){
    RM_CALCPOST( DAT2=dat2, DAT2RESP=dat2resp, PROBS=probs, KK=K)
}
