## File Name: RcppExports.R
## File Version: 4.002133
# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

gooijer_csn_table <- function(dat, dat_perm, RR, NS, progress, progress_vec, score_index) {
    .Call('_sirt_gooijer_csn_table', PACKAGE='sirt', dat, dat_perm, RR, NS, progress, progress_vec, score_index)
}

isop_tests_C <- function(dat, dat_resp, weights, jackunits, JJ) {
    .Call('_sirt_isop_tests_C', PACKAGE='sirt', dat, dat_resp, weights, jackunits, JJ)
}

interval_index_C <- function(MATR, RN) {
    .Call('_sirt_interval_index_C', PACKAGE='sirt', MATR, RN)
}

rowCumsums2_source <- function(input) {
    .Call('_sirt_rowCumsums2_source', PACKAGE='sirt', input)
}

rowKSmallest_C <- function(MATR, KK1, INDEXMATR, RNMATR) {
    .Call('_sirt_rowKSmallest_C', PACKAGE='sirt', MATR, KK1, INDEXMATR, RNMATR)
}

rowMaxsCPP_source <- function(MATR) {
    .Call('_sirt_rowMaxsCPP_source', PACKAGE='sirt', MATR)
}

rowmins2_bundle_C <- function(m1, v1) {
    .Call('_sirt_rowmins2_bundle_C', PACKAGE='sirt', m1, v1)
}

calc_copula_itemcluster_C <- function(DD, res) {
    .Call('_sirt_calc_copula_itemcluster_C', PACKAGE='sirt', DD, res)
}

md_pattern_csource <- function(dat) {
    .Call('_sirt_md_pattern_csource', PACKAGE='sirt', dat)
}

mle_pcm_group_C <- function(dat, dat_resp, groupM, b, a, maxK, theta0, conv, maxiter) {
    .Call('_sirt_mle_pcm_group_C', PACKAGE='sirt', dat, dat_resp, groupM, b, a, maxK, theta0, conv, maxiter)
}

mle_pcm_C <- function(dat, dat_resp, b, a, maxK, theta0, conv, maxiter) {
    .Call('_sirt_mle_pcm_C', PACKAGE='sirt', dat, dat_resp, b, a, maxK, theta0, conv, maxiter)
}

probs_pcm_groups_C <- function(dat, dat_resp, group, probs, CC, TP) {
    .Call('_sirt_probs_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, probs, CC, TP)
}

sirt_rcpp_probs_pcm_groups_C <- function(dat, dat_resp, group, probs, CC, TP) {
    .Call('_sirt_sirt_rcpp_probs_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, probs, CC, TP)
}

probs_pcm_nogroups_C <- function(dat, dat_resp, probs, CC, TP) {
    .Call('_sirt_probs_pcm_nogroups_C', PACKAGE='sirt', dat, dat_resp, probs, CC, TP)
}

calccounts_pcm_groups_C <- function(dat, dat_resp, group, fyiqk, pik, CC, weights) {
    .Call('_sirt_calccounts_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, fyiqk, pik, CC, weights)
}

sirt_rcpp_calccounts_pcm_groups_C <- function(dat, dat_resp, group, fyiqk, pik, CC, weights) {
    .Call('_sirt_sirt_rcpp_calccounts_pcm_groups_C', PACKAGE='sirt', dat, dat_resp, group, fyiqk, pik, CC, weights)
}

sirt_rcpp_array_mult <- function(AA, dimAA, BB, dimBB) {
    .Call('_sirt_sirt_rcpp_array_mult', PACKAGE='sirt', AA, dimAA, BB, dimBB)
}

RM_CALCPOST <- function(DAT2, DAT2RESP, PROBS, KK) {
    .Call('_sirt_RM_CALCPOST', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, KK)
}

rm_probraterfct1 <- function(CRA, DRA, dimAA, BB, dimBB) {
    .Call('_sirt_rm_probraterfct1', PACKAGE='sirt', CRA, DRA, dimAA, BB, dimBB)
}

rm_facets_calcprobs_cpp <- function(b_item, b_rater, Qmatrix, tau_item, K, I, TP, a_item, a_rater, item_index, rater_index, theta_k) {
    .Call('_sirt_rm_facets_calcprobs_cpp', PACKAGE='sirt', b_item, b_rater, Qmatrix, tau_item, K, I, TP, a_item, a_rater, item_index, rater_index, theta_k)
}

SMIRT_CALCPOST <- function(DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1) {
    .Call('_sirt_SMIRT_CALCPOST', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1)
}

SMIRT_CALCPROB_COMP <- function(A, B, QQ, THETA, CC, DD) {
    .Call('_sirt_SMIRT_CALCPROB_COMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD)
}

SMIRT_CALCPROB_NONCOMP <- function(A, B, QQ, THETA, CC, DD) {
    .Call('_sirt_SMIRT_CALCPROB_NONCOMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD)
}

SMIRT_CALCPROB_PARTCOMP <- function(A, B, QQ, THETA, CC, DD, MUI) {
    .Call('_sirt_SMIRT_CALCPROB_PARTCOMP', PACKAGE='sirt', A, B, QQ, THETA, CC, DD, MUI)
}

MML2_RASCHTYPE_COUNTS <- function(DAT2, DAT2RESP, DAT1, FQKYI, PIK, FYIQK) {
    .Call('_sirt_MML2_RASCHTYPE_COUNTS', PACKAGE='sirt', DAT2, DAT2RESP, DAT1, FQKYI, PIK, FYIQK)
}

MML2_CALCPOST_V1 <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_MML2_CALCPOST_V1', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

sirt_rcpp_rasch_mml2_calcpost_pseudoll <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_sirt_rcpp_rasch_mml2_calcpost_pseudoll', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

MML2_CALCPOST_V3 <- function(DAT2, DAT2RESP, PROBS) {
    .Call('_sirt_MML2_CALCPOST_V3', PACKAGE='sirt', DAT2, DAT2RESP, PROBS)
}

sirt_rcpp_ccov_np_compute_ccov_sum_score <- function(index, NS, data) {
    .Call('_sirt_sirt_rcpp_ccov_np_compute_ccov_sum_score', PACKAGE='sirt', index, NS, data)
}

sirt_rcpp_discrete_inverse <- function(x0, y0, y) {
    .Call('_sirt_sirt_rcpp_discrete_inverse', PACKAGE='sirt', x0, y0, y)
}

sirt_rcpp_first_eigenvalue <- function(X, maxit, conv, K) {
    .Call('_sirt_sirt_rcpp_first_eigenvalue', PACKAGE='sirt', X, maxit, conv, K)
}

sirt_rcpp_D_eigenvalues <- function(Xr, D, maxit, conv) {
    .Call('_sirt_sirt_rcpp_D_eigenvalues', PACKAGE='sirt', Xr, D, maxit, conv)
}

sirt_rcpp_choppin_row_averaging <- function(B, I, priorweight) {
    .Call('_sirt_sirt_rcpp_choppin_row_averaging', PACKAGE='sirt', B, I, priorweight)
}

sirt_rcpp_evm_compute <- function(B, I, powD, maxit, conv, K) {
    .Call('_sirt_sirt_rcpp_evm_compute', PACKAGE='sirt', B, I, powD, maxit, conv, K)
}

sirt_rcpp_evm_comp_poly <- function(dat, dat_resp, weights, JJ, jackunits, powD, progress_, row_index, col_index) {
    .Call('_sirt_sirt_rcpp_evm_comp_poly', PACKAGE='sirt', dat, dat_resp, weights, JJ, jackunits, powD, progress_, row_index, col_index)
}

sirt_rcpp_gom_em_calcpost <- function(DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_calcpost', PACKAGE='sirt', DAT2, DAT2RESP, PROBS, DAT2IND, PIK, KK1, weights)
}

sirt_rcpp_gom_em_likelihood <- function(probs, ncat, TP, dat2, dat2resp) {
    .Call('_sirt_sirt_rcpp_gom_em_likelihood', PACKAGE='sirt', probs, ncat, TP, dat2, dat2resp)
}

sirt_rcpp_gom_em_log_likelihood <- function(fyiqk, pik, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_log_likelihood', PACKAGE='sirt', fyiqk, pik, weights)
}

sirt_rcpp_gom_em_loglike_gradient <- function(probs, probs_h, ncat, TP, dat2, dat2resp, pik, items, fyiqk, weights) {
    .Call('_sirt_sirt_rcpp_gom_em_loglike_gradient', PACKAGE='sirt', probs, probs_h, ncat, TP, dat2, dat2resp, pik, items, fyiqk, weights)
}

sirt_rcpp_inference_jackknife <- function(PARS) {
    .Call('_sirt_sirt_rcpp_inference_jackknife', PACKAGE='sirt', PARS)
}

sirt_rcpp_invariance_alignment_lambda_transformed <- function(lambda, psi0, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_lambda_transformed', PACKAGE='sirt', lambda, psi0, meth)
}

sirt_rcpp_invariance_alignment_nu_transformed <- function(nu, lambda, alpha0, psi0, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_nu_transformed', PACKAGE='sirt', nu, lambda, alpha0, psi0, reparam, meth)
}

sirt_rcpp_invariance_alignment_simplicity_function_value <- function(type, parm1, parm2, scale, power, eps) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_simplicity_function_value', PACKAGE='sirt', type, parm1, parm2, scale, power, eps)
}

sirt_rcpp_invariance_alignment_opt_fct <- function(nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_opt_fct', PACKAGE='sirt', nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth)
}

sirt_rcpp_invariance_alignment_simplicity_function_gradient <- function(type, parm1, parm2, scale, power, eps) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_simplicity_function_gradient', PACKAGE='sirt', type, parm1, parm2, scale, power, eps)
}

sirt_rcpp_invariance_alignment_opt_grad <- function(nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth) {
    .Call('_sirt_sirt_rcpp_invariance_alignment_opt_grad', PACKAGE='sirt', nu, lambda, alpha0, psi0, group_combis, wgt, align_scale, align_pow, eps, wgt_combi, type, reparam, meth)
}

sirt_rcpp_linking_haebara_fct_optim_one_item <- function(theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_fct_optim_one_item', PACKAGE='sirt', theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, pow)
}

sirt_rcpp_linking_haebara_fct_optim <- function(NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_fct_optim', PACKAGE='sirt', NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, pow)
}

sirt_rcpp_linking_haebara_grad_optim_one_item <- function(theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, NI, NS, index_a, index_b, index_mu, index_sigma, wgtM, grad0, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_grad_optim_one_item', PACKAGE='sirt', theta, prob_theta, aM, bM, a, b, mu, sigma, ii, ss, dist, eps, NI, NS, index_a, index_b, index_mu, index_sigma, wgtM, grad0, pow)
}

sirt_rcpp_linking_haebara_grad_optim <- function(NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, index_a, index_b, index_mu, index_sigma, parnames, NP, pow) {
    .Call('_sirt_sirt_rcpp_linking_haebara_grad_optim', PACKAGE='sirt', NI, NS, dist, aM, bM, theta, prob_theta, est_pars, wgtM, a, b, mu, sigma, eps, index_a, index_b, index_mu, index_sigma, parnames, NP, pow)
}

sirt_rcpp_lq_fit_analyze_matrix <- function(X) {
    .Call('_sirt_sirt_rcpp_lq_fit_analyze_matrix', PACKAGE='sirt', X)
}

sirt_rcpp_lq_fit_sparse_matrix_derivative <- function(Z, h1, px) {
    .Call('_sirt_sirt_rcpp_lq_fit_sparse_matrix_derivative', PACKAGE='sirt', Z, h1, px)
}

sirt_rcpp_lq_fit_matrix_mult <- function(Z, y, beta) {
    .Call('_sirt_sirt_rcpp_lq_fit_matrix_mult', PACKAGE='sirt', Z, y, beta)
}

sirt_rcpp_lq_fit_fct_optim <- function(Z, y, beta, pow, w, eps) {
    .Call('_sirt_sirt_rcpp_lq_fit_fct_optim', PACKAGE='sirt', Z, y, beta, pow, w, eps)
}

sirt_rcpp_mgsem_quadform <- function(y, A) {
    .Call('_sirt_sirt_rcpp_mgsem_quadform', PACKAGE='sirt', y, A)
}

sirt_rcpp_mgsem_quadform_logical <- function(y, A, A_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_quadform_logical', PACKAGE='sirt', y, A, A_logical)
}

sirt_rcpp_mgsem_trace_product <- function(A, B) {
    .Call('_sirt_sirt_rcpp_mgsem_trace_product', PACKAGE='sirt', A, B)
}

sirt_rcpp_mgsem_trace_product_logical <- function(A, B, B_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_trace_product_logical', PACKAGE='sirt', A, B, B_logical)
}

sirt_rcpp_mgsem_loglike_derivative_parameters <- function(S1, S3, y, Sigma_der, Sigma_der_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_loglike_derivative_parameters', PACKAGE='sirt', S1, S3, y, Sigma_der, Sigma_der_logical)
}

sirt_rcpp_mgsem_compute_cov <- function(LAM, PHI, PSI) {
    .Call('_sirt_sirt_rcpp_mgsem_compute_cov', PACKAGE='sirt', LAM, PHI, PSI)
}

sirt_rcpp_mgsem_sumproduct_logical <- function(x, y, y_logical) {
    .Call('_sirt_sirt_rcpp_mgsem_sumproduct_logical', PACKAGE='sirt', x, y, y_logical)
}

sirt_rcpp_mgsem_vech_numeric <- function(A) {
    .Call('_sirt_sirt_rcpp_mgsem_vech_numeric', PACKAGE='sirt', A)
}

sirt_rcpp_mgsem_vech_logical <- function(A) {
    .Call('_sirt_sirt_rcpp_mgsem_vech_logical', PACKAGE='sirt', A)
}

sirt_rcpp_mgsem_eval_pen_lp_lasso <- function(y, eps_approx, p, regul_fac) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_lasso', PACKAGE='sirt', y, eps_approx, p, regul_fac)
}

sirt_rcpp_mgsem_eval_pen_lp_scad <- function(y, eps_approx, p, regul_fac, a_scad) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_scad', PACKAGE='sirt', y, eps_approx, p, regul_fac, a_scad)
}

sirt_rcpp_mgsem_eval_pen_lp_lasso_deriv <- function(y, eps_approx, p, regul_fac) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_pen_lp_lasso_deriv', PACKAGE='sirt', y, eps_approx, p, regul_fac)
}

sirt_rcpp_mgsem_eval_lp_penalty <- function(x, fac, n, p, eps_approx, deriv, pen_type, a_scad, h) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lp_penalty', PACKAGE='sirt', x, fac, n, p, eps_approx, deriv, pen_type, a_scad, h)
}

sirt_rcpp_mgsem_eval_lpdiff_penalty <- function(x, fac, fac_logical, p, eps_approx, a_scad, pen_type, n) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lpdiff_penalty', PACKAGE='sirt', x, fac, fac_logical, p, eps_approx, a_scad, pen_type, n)
}

sirt_rcpp_mgsem_eval_lpdiff_penalty_deriv <- function(x, fac, n, fac_logical, p, eps_approx, h, a_scad, pen_type) {
    .Call('_sirt_sirt_rcpp_mgsem_eval_lpdiff_penalty_deriv', PACKAGE='sirt', x, fac, n, fac_logical, p, eps_approx, h, a_scad, pen_type)
}

sirt_rcpp_monoreg_rowwise <- function(YM, WM) {
    .Call('_sirt_sirt_rcpp_monoreg_rowwise', PACKAGE='sirt', YM, WM)
}

sirt_rcpp_noharm_sirt_optim_fn_rcpp <- function(gamma_val, delta, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk) {
    .Call('_sirt_sirt_rcpp_noharm_sirt_optim_fn_rcpp', PACKAGE='sirt', gamma_val, delta, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk)
}

sirt_rcpp_noharm_sirt_optim_gr_rcpp <- function(gamma_val, NH, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk, npar, pt_matid, pt_index, pt_row, pt_col, FP, Fmat, Pmat, Psimat) {
    .Call('_sirt_sirt_rcpp_noharm_sirt_optim_gr_rcpp', PACKAGE='sirt', gamma_val, NH, I, wgtm, pm, b0_jk, b1_jk, b2_jk, b3_jk, npar, pt_matid, pt_index, pt_row, pt_col, FP, Fmat, Pmat, Psimat)
}

sirt_rcpp_pnorm0 <- function(z) {
    .Call('_sirt_sirt_rcpp_pnorm0', PACKAGE='sirt', z)
}

sirt_rcpp_pnorm <- function(x) {
    .Call('_sirt_sirt_rcpp_pnorm', PACKAGE='sirt', x)
}

sirt_rcpp_pbvnorm0 <- function(h1, hk, r) {
    .Call('_sirt_sirt_rcpp_pbvnorm0', PACKAGE='sirt', h1, hk, r)
}

sirt_rcpp_pbvnorm <- function(x, y, rho) {
    .Call('_sirt_sirt_rcpp_pbvnorm', PACKAGE='sirt', x, y, rho)
}

sirt_rcpp_dbvnorm0 <- function(x, y, rho, use_log) {
    .Call('_sirt_sirt_rcpp_dbvnorm0', PACKAGE='sirt', x, y, rho, use_log)
}

sirt_rcpp_dbvnorm <- function(x, y, rho, use_log) {
    .Call('_sirt_sirt_rcpp_dbvnorm', PACKAGE='sirt', x, y, rho, use_log)
}

sirt_rcpp_pbivnorm2 <- function(x, y, rho1) {
    .Call('_sirt_sirt_rcpp_pbivnorm2', PACKAGE='sirt', x, y, rho1)
}

sirt_rcpp_dmvnorm_2dim <- function(x, y, rho) {
    .Call('_sirt_sirt_rcpp_dmvnorm_2dim', PACKAGE='sirt', x, y, rho)
}

sirt_rcpp_polychoric2_pbivnorm <- function(x, y, rho, use_pbv) {
    .Call('_sirt_sirt_rcpp_polychoric2_pbivnorm', PACKAGE='sirt', x, y, rho, use_pbv)
}

sirt_rcpp_polychoric2_estimating_equation <- function(frtab, maxK, rho, thresh1n, thresh2n, maxK1, maxK2, use_pbv) {
    .Call('_sirt_sirt_rcpp_polychoric2_estimating_equation', PACKAGE='sirt', frtab, maxK, rho, thresh1n, thresh2n, maxK1, maxK2, use_pbv)
}

sirt_rcpp_polychoric2_est_itempair <- function(v1, v2, maxK_, maxiter, use_pbv, conv, rho_init, weights) {
    .Call('_sirt_sirt_rcpp_polychoric2_est_itempair', PACKAGE='sirt', v1, v2, maxK_, maxiter, use_pbv, conv, rho_init, weights)
}

sirt_rcpp_polychoric2 <- function(dat, maxK, maxiter, use_pbv, conv, rho_init, weights) {
    .Call('_sirt_sirt_rcpp_polychoric2', PACKAGE='sirt', dat, maxK, maxiter, use_pbv, conv, rho_init, weights)
}

sirt_rcpp_tetrachoric2_rcpp <- function(dfr, h, maxiter) {
    .Call('_sirt_sirt_rcpp_tetrachoric2_rcpp', PACKAGE='sirt', dfr, h, maxiter)
}

sirt_rcpp_rm_proc_datasets_na_indicators <- function(dat, K) {
    .Call('_sirt_sirt_rcpp_rm_proc_datasets_na_indicators', PACKAGE='sirt', dat, K)
}

sirt_rcpp_rm_proc_expand_dataset <- function(dat, rater0, pid0, N, R) {
    .Call('_sirt_sirt_rcpp_rm_proc_expand_dataset', PACKAGE='sirt', dat, rater0, pid0, N, R)
}

sirt_rcpp_rm_sdt_posterior <- function(prob_item, prob_rater, I0, K1, TP, item_index, dat2, dat2_resp, I, pi_k0) {
    .Call('_sirt_sirt_rcpp_rm_sdt_posterior', PACKAGE='sirt', prob_item, prob_rater, I0, K1, TP, item_index, dat2, dat2_resp, I, pi_k0)
}

sirt_rcpp_rm_sdt_calc_gradient_item_deriv_a <- function(prob0, prob_D1_dim, theta_k) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_item_deriv_a', PACKAGE='sirt', prob0, prob_D1_dim, theta_k)
}

sirt_rcpp_rm_sdt_calc_gradient_item_deriv_tau <- function(prob0, prob_D1_dim, cat_pp) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_item_deriv_tau', PACKAGE='sirt', prob0, prob_D1_dim, cat_pp)
}

sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad <- function(logprob_D1, prob_D1_dim, nik_item) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_gradient_likelihood_item_llgrad', PACKAGE='sirt', logprob_D1, prob_D1_dim, nik_item)
}

sirt_rcpp_squeeze_eps <- function(x, eps) {
    .Call('_sirt_sirt_rcpp_squeeze_eps', PACKAGE='sirt', x, eps)
}

sirt_rcpp_log <- function(x) {
    .Call('_sirt_sirt_rcpp_log', PACKAGE='sirt', x)
}

sirt_rcpp_plogis <- function(x) {
    .Call('_sirt_sirt_rcpp_plogis', PACKAGE='sirt', x)
}

sirt_rcpp_rm_sdt_calc_probs_gpcm <- function(a, tau, theta_k, VV, K1, TP, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_gpcm', PACKAGE='sirt', a, tau, theta_k, VV, K1, TP, eps, use_log)
}

sirt_rcpp_rm_sdt_calc_probs_grm_rater <- function(c_rater, d_rater, I, K, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_grm_rater', PACKAGE='sirt', c_rater, d_rater, I, K, eps, use_log)
}

sirt_rcpp_rm_sdt_calc_probs_grm_item <- function(tau_item, a_item, theta_k, VV, K, TP, eps, use_log) {
    .Call('_sirt_sirt_rcpp_rm_sdt_calc_probs_grm_item', PACKAGE='sirt', tau_item, a_item, theta_k, VV, K, TP, eps, use_log)
}

sirt_rcpp_xxirt_compute_posterior_expected_counts <- function(dat1_resp_gg, p_aj_xi_gg, weights_gg) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_posterior_expected_counts', PACKAGE='sirt', dat1_resp_gg, p_aj_xi_gg, weights_gg)
}

sirt_rcpp_xxirt_compute_likelihood <- function(dat, dat_resp_bool, probs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_likelihood', PACKAGE='sirt', dat, dat_resp_bool, probs, TP, maxK)
}

sirt_rcpp_xxirt_compute_likelihood_person_covariates <- function(dat, dat_resp_bool, probs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_likelihood_person_covariates', PACKAGE='sirt', dat, dat_resp_bool, probs, TP, maxK)
}

sirt_rcpp_xxirt_compute_loglike_case_theta_covariates <- function(dat, dat_resp_bool, logprobs, TP, maxK) {
    .Call('_sirt_sirt_rcpp_xxirt_compute_loglike_case_theta_covariates', PACKAGE='sirt', dat, dat_resp_bool, logprobs, TP, maxK)
}

sirt_rcpp_xxirt_hessian_reduced_probs <- function(dat, dat_resp_bool, probs_ratio, TP, maxK, itemnr, itemnr2, use_itemnr2, p_xi_aj) {
    .Call('_sirt_sirt_rcpp_xxirt_hessian_reduced_probs', PACKAGE='sirt', dat, dat_resp_bool, probs_ratio, TP, maxK, itemnr, itemnr2, use_itemnr2, p_xi_aj)
}

sirt_rcpp_xxirt_newton_raphson_derivative_par <- function(dat, dat_resp_bool, ratio, p_xi_aj, item, prior_Theta, group0, weights, ll_case0, eps, person_covariates, person_covariates_items) {
    .Call('_sirt_sirt_rcpp_xxirt_newton_raphson_derivative_par', PACKAGE='sirt', dat, dat_resp_bool, ratio, p_xi_aj, item, prior_Theta, group0, weights, ll_case0, eps, person_covariates, person_covariates_items)
}

sirt_rcpp_xxirt_nr_pml_opt_fun <- function(prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_opt_fun', PACKAGE='sirt', prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps)
}

sirt_rcpp_xxirt_nr_pml_grad_fun_eval <- function(prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps, NP, der_prior_Theta, val1, val2, pp_Theta, der_probs_items, index_freq1, index_freq2) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_grad_fun_eval', PACKAGE='sirt', prior_Theta, probs_items, freq1, freq2, W1, W2_long, G, K, I, TP, NI2, eps, NP, der_prior_Theta, val1, val2, pp_Theta, der_probs_items, index_freq1, index_freq2)
}

sirt_rcpp_xxirt_nr_pml_casewise_opt_fun <- function(prior_Theta, probs_items, W1, W2_long, G, K, I, TP, NI2, eps, group0, weights, dat1, dat_resp) {
    .Call('_sirt_sirt_rcpp_xxirt_nr_pml_casewise_opt_fun', PACKAGE='sirt', prior_Theta, probs_items, W1, W2_long, G, K, I, TP, NI2, eps, group0, weights, dat1, dat_resp)
}

