## File Name: lsem_permutationTest_define_exported_objects.R
## File Version: 0.105

lsem_permutationTest_define_exported_objects <- function()
{
    varlist <- c('arglist', 'lsem_permutationTest_collect_output')
    return(varlist)
}
