%% File Name: data.liking.science.Rd
%% File Version: 0.08

\name{data.liking.science}
\alias{data.liking.science}
\docType{data}

\title{
Dataset 'Liking For Science'
}

\description{
Dataset 'Liking for science' published by Wright and Masters (1982).
}

\usage{
data(data.liking.science)
}

\format{
The format is:

\code{ num [1:75, 1:24] 1 2 2 1 1 1 2 2 0 2 ...} \cr
\code{ - attr(*, "dimnames")=List of 2} \cr
\code{  ..$ : NULL} \cr
\code{  ..$ : chr [1:24] "LS01" "LS02" "LS03" "LS04" ...} \cr
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}



\references{
Wright, B. D., & Masters, G. N. (1982). \emph{Rating scale analysis}.
Chicago: MESA Press.
}
%\examples{
%data(data.liking.science)
%## maybe str(data.liking.science) ; plot(data.liking.science) ...
%}
%% \keyword{datasets}
