%% File Name: data.inv4gr.Rd
%% File Version: 0.08

\name{data.inv4gr}
\alias{data.inv4gr}
\docType{data}
\title{
Dataset for Invariance Testing with 4 Groups
}
\description{
Dataset for invariance testing with 4 groups.
}
\usage{
data(data.inv4gr)
}

\format{
  A data frame with 4000 observations on the following 12 variables.
  The first variable is a group identifier, the other variables are items.
  \describe{
    \item{\code{group}}{A group identifier}
    \item{\code{I01}}{a numeric vector}
    \item{\code{I02}}{a numeric vector}
    \item{\code{I03}}{a numeric vector}
    \item{\code{I04}}{a numeric vector}
    \item{\code{I05}}{a numeric vector}
    \item{\code{I06}}{a numeric vector}
    \item{\code{I07}}{a numeric vector}
    \item{\code{I08}}{a numeric vector}
    \item{\code{I09}}{a numeric vector}
    \item{\code{I10}}{a numeric vector}
    \item{\code{I11}}{a numeric vector}
  }
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}

\source{
Simulated dataset
}

%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



%\examples{
%data(data.inv4gr)
%## maybe str(data.inv4gr) ; plot(data.inv4gr) ...
%}
%% \keyword{datasets}
