%% File Name: data.pw01.Rd
%% File Version: 0.06

\name{data.pw}
\alias{data.pw01}
\docType{data}
\title{
Datasets for Pairwise Comparisons
}
\description{
Some datasets for pairwise comparisons.
}
\usage{
data(data.pw01)
}
\format{
The dataset \code{data.pw01} contains results of a German football league
from the season 2000/01.
}

%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}
%\examples{
%data(data.pw01)
%}
%% \keyword{datasets}
