%% File Name: data.noharm.Rd
%% File Version: 2.11

\name{data.noharm}


\alias{data.noharm18}
\alias{data.noharmExC}


\docType{data}
\title{
Datasets for NOHARM Analysis
}
\description{
Datasets for analyses in NOHARM (see \code{\link{R2noharm}}).
}
\usage{
data(data.noharmExC)
data(data.noharm18)
}


\format{

\itemize{

\item  \code{data.noharmExC} \cr

The format of this dataset is

\code{'data.frame':   300 obs. of  8 variables:} \cr
\code{ $ C1: int  1 1 1 1 1 0 1 1 1 1 ...} \cr
\code{ $ C2: int  1 1 1 1 0 1 1 1 1 1 ...} \cr
\code{ $ C3: int  1 1 1 1 1 0 0 0 1 1 ...} \cr
\code{ $ C4: int  0 0 1 1 1 1 1 0 1 0 ...} \cr
\code{ $ C5: int  1 1 1 1 1 0 0 1 1 0 ...} \cr
\code{ $ C6: int  1 0 0 0 1 0 1 1 0 1 ...} \cr
\code{ $ C7: int  1 1 0 0 1 1 0 0 0 1 ...} \cr
\code{ $ C8: int  1 0 1 0 1 0 1 0 1 1 ...} \cr


\item \code{data.noharm18} \cr

A data frame with 200 observations on the following 18 variables \code{I01},
  ..., \code{I18}. The format is

\code{'data.frame':   200 obs. of  18 variables:} \cr
\code{ $ I01: int  1 1 1 1 1 0 1 1 0 1 ...} \cr
\code{ $ I02: int  1 1 0 1 1 0 1 1 1 1 ...} \cr
\code{ $ I03: int  1 0 0 1 0 0 1 1 0 1 ...} \cr
\code{ $ I04: int  0 1 0 1 0 0 0 1 1 1 ...} \cr
\code{ $ I05: int  1 0 0 0 1 0 1 1 0 1 ...} \cr
\code{ $ I06: int  1 1 0 1 0 0 1 1 0 1 ...} \cr
\code{ $ I07: int  1 1 1 1 0 1 1 1 1 1 ...} \cr
\code{ $ I08: int  1 1 1 1 1 1 1 1 0 1 ...} \cr
\code{ $ I09: int  1 1 1 1 0 0 1 1 0 1 ...} \cr
\code{ $ I10: int  1 0 0 1 1 0 1 1 0 1 ...} \cr
\code{ $ I11: int  1 1 1 1 0 0 1 1 0 1 ...} \cr
\code{ $ I12: int  0 0 0 0 0 1 0 0 0 0 ...} \cr
\code{ $ I13: int  1 1 1 1 0 1 1 0 1 1 ...} \cr
\code{ $ I14: int  1 1 1 0 1 0 1 1 0 1 ...} \cr
\code{ $ I15: int  1 1 1 0 0 1 1 1 0 1 ...} \cr
\code{ $ I16: int  1 1 0 1 1 0 1 0 1 1 ...} \cr
\code{ $ I17: int  0 1 0 0 0 0 1 1 0 1 ...} \cr
\code{ $ I18: int  0 0 0 0 0 0 0 0 1 0 ...} \cr
}
}
%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}



%\examples{
%data(data.noharm)
%## maybe str(data.noharm) ; plot(data.noharm) ...
%}
%% \keyword{datasets}
