%% File Name: data.raw1.Rd
%% File Version: 0.07

\name{data.raw1}
\alias{data.raw1}
\docType{data}
\title{
Dataset with Raw Item Responses
}
\description{
Dataset with raw item responses
}
\usage{data(data.raw1)}
\format{
A data frame with raw item responses of 1200 persons on the following 77 items:

\code{'data.frame':   1200 obs. of  77 variables:} \cr
\code{ $ I101: num  0 0 0 2 0 0 0 0 0 0 ...} \cr
\code{ $ I102: int  NA NA 2 1 2 1 3 2 NA NA ...} \cr
\code{ $ I103: int  1 1 NA NA NA NA NA NA 1 1 ...} \cr
\code{ ...} \cr
\code{ $ I179: chr  "E" "C" "D" "E" ...} \cr

}
%\details{
%%  ~~ If necessary, more details than the __description__ above ~~
%}
%\source{
%%  ~~ reference to a publication or URL from which the data were obtained ~~
%}
%\references{
%%  ~~ possibly secondary sources and usages ~~
%}




%\examples{
%data(data.raw1)
%## maybe str(data.raw1) ; plot(data.raw1) ...
%}
%% \keyword{datasets}
